freeze;

Z2 := Integers(2);
Z4 := Integers(4);
C2 := AbelianGroup([2]);



//////////////////////////////////////////////////
// Level

intrinsic Level( K::FldFin ) -> RngIntElt
{    
This function comutes the level s(K) of a number field K,
i.e. a lowest number s such that -1 is a sum of s squares
it returns 0 if K is formally real.
}
    s := case< #K mod 4 | 3 : 2, default: 1 >;
    assert2 s eq LengthOfSOS(K!-1);
    return s;
end intrinsic;

//////////////////////////////////////////////////
// IsFormallyReal
// just for completeness
intrinsic IsFormallyReal( K::FldRat ) -> BoolElt
{
Returns true iff K is formally real (i.e. -1 is not a sum of squares)
}
    return false;
end intrinsic;

intrinsic IsReal( K::FldRat ) -> BoolElt
{ " } //"
    return false;
end intrinsic;


//////////////////////////////////////////////////
// PythagorasNumber & PythagorasElement

intrinsic PythagorasNumber( K::FldFin ) -> RngIntElt
{
This function computes the Pythagoras number P(K) of a field K,
i.e. the lowest number p such that every sum of squares in K
is a sum of p squares.
}
    // Use [Szymiczek (1997), Exm. 18.2.1] and
    // [Lam (2005), Prop. II.3.4]
    return IsOdd(Characteristic(K)) select 2 else 1;
end intrinsic;

intrinsic PythagorasElement( K::FldFin ) -> FldNumElt
{
Returns an element a of K whose length equals the Pythagoras 
number P(K) of K.
}
    return IsOdd(Characteristic(K)) select Nonsquare(K) else One(K);
end intrinsic;

//////////////////////////////////////////////////
// Witt ring
intrinsic WittRing( F::FldFin ) -> RngWittQF
{
Constructs the Witt ring of similarity classes of quadratic forms over F.
}
    if not assigned F`Witt_ring then
        W := New(RngWittQF);
        W`base_ring := F;
        W`is_explicit := true;
        if #F mod 4 eq 1 then
            // Use [Szymiczek (1997), Exc. 13.3.5]
            W`explicit_model := GroupAlgebra(Z2, C2);
        elif #F mod 4 eq 3 then
            // Use [Szymiczek (1997), Exc. 13.3.5]
            W`explicit_model, _ := Z4;
        elif Characteristic(F) eq 2 then
            // Use [Szymiczek (1997), Prop. 13.2.3]
            W`explicit_model, _ := Z2;
        else 
            require false: "The base field is not supported!";
        end if;
        F`Witt_ring := W;
    end if;
    return F`Witt_ring;
end intrinsic;

