freeze;

declare attributes FldFin: Witt_ring;

//////////////////////////////////////////////////
// quadratic forms

function is_isotropic_FF( q )
    // Let q be a diagonal quadratic form with coefficients in
    // a finite field of odd characteristic. This function checks
    // if q is isotropic
    
    // the base field
    F := q`base_ring;
    assert IsFinite(F);
    assert Characteristic(F) ne 2;
    
    d := Dimension(q);
    if d le 1 then
        return false;
    elif d eq 2 then
        t := IsSquare(Discriminant(q));
        return t;
    end if;
    return true;
end function;

function is_hyperbolic_FF( q )
    // Let q be a diagonal quadratic form with coefficients in
    // a finite field of odd characteristic. This function checks
    // if q is hyperbolic
    
    // the base field
    F := q`base_ring;
    assert IsFinite(F);
    assert Characteristic(F) ne 2;
    
    d := Dimension(q);
    if IsOdd(d) then
        return false;
    end if;
    if IsSquare(Discriminant(q)) then
        return true;
    end if;
    return false;
end function;

function anisotropic_dimension_FF( q )
    if IsOdd(Dimension(q)) then
        return 1;
    elif IsHyperbolic(q) then
        return 0;
    else
        return 2;
    end if;
end function;


//////////////////////////////////////////////////
// IsTorsion
// just for completeness
function is_torsion_FF( _q )
// Check if the element _q of a Witt ring is torsion.
    return true;
end function;
