freeze;

//////////////////////////////////////////////////
// Level
// (just for completeness)
intrinsic Level( K::FldFunRat ) -> RngIntElt
{    
This function comutes the level s(K) of a number field K,
i.e. a lowest number s such that -1 is a sum of s squares
it returns 0 if K is formally real.
}
    return Level( ConstantField(K) );
end intrinsic;


//////////////////////////////////////////////////
// IsFormallyReal
// just for completeness
intrinsic IsFormallyReal( K::FldFunRat ) -> BoolElt
{
Returns true iff K is formally real (i.e. -1 is not a sum of squares)
}
    return IsFormallyReal( ConstantField(K) );
end intrinsic;

intrinsic IsReal( K::FldFunRat ) -> BoolElt
{ " } //"
    return IsFormallyReal(K);
end intrinsic;


//////////////////////////////////////////////////
// PythagorasNumber
// just for completeness
intrinsic PythagorasNumber( K::FldFunRat[FldFin] ) -> RngIntElt
{
This function computes the Pythagoras number P(K) of a number field K,
i.e. the lowest number p such that every sum of squares in K
is a sum of p squares.
}
    return 1 + PythagorasNumber( ConstantField(K) );
end intrinsic;

intrinsic PythagorasElement( K::FldFunRat[FldFin] ) -> FldNumElt
{
Returns an element a of K whose length equals the Pythagoras 
number P(K) of K.
}
    return K.1^2 + PythagorasElement( ConstantField(K) );
end intrinsic;

//////////////////////////////////////////////////
// AreWittEquivalent

intrinsic AreWittEquivalent( K::FldFunRat, L::FldFunRat ) -> BoolElt
{
Given two fields K and L, this function return true if their Witt rings are isomorphic and false otherwise.
}    
    require ISA(Category(ConstantField(K)), FldFin) : "Not implemented for these types of fields!";
    require ISA(Category(ConstantField(L)), FldFin) : "Not implemented for these types of fields!";

    // Use Szymiczek (1991), Theorem 1.1
    if Characteristic(K) eq 2 and Characteristic(L) ne 2 then
        return false;
    elif Characteristic(K) ne 2 and Characteristic(L) eq 2 then
        return false;
    elif Characteristic(K) eq 2 and Characteristic(L) eq 2 then
        return true;
    else
        // Now both must have odd characteristic
        // Use Szymiczek (1991), Theorem 1.3
        return Level(K) eq Level(L);
    end if;
end intrinsic;

intrinsic AreWittEquivalent( K::FldFunRat, L::FldFun ) -> BoolElt
{ " } //"
    require ISA(Category(ConstantField(K)), FldFin) : "Not implemented for these types of fields!";
    require ISA(Category(ConstantField(L)), FldFin) : "Not implemented for these types of fields!";

    // Use Szymiczek (1991), Theorem 1.1
    if Characteristic(K) eq 2 and Characteristic(L) ne 2 then
        return false;
    elif Characteristic(K) ne 2 and Characteristic(L) eq 2 then
        return false;
    elif Characteristic(K) eq 2 and Characteristic(L) eq 2 then
        return true;
    else
        // Now both must have odd characteristic
        // Use Szymiczek (1991), Theorem 1.3
        return Level(K) eq Level(L);
    end if;
end intrinsic;

intrinsic AreWittEquivalent( K::FldFunRat, L::FldNum ) -> BoolElt
{ " } //"
    require ISA(Category(ConstantField(K)), FldFin) : "Not implemented for these types of fields!";

    // Use Szymiczek (1991), Theorem 1.3
    return false;
end intrinsic;

intrinsic AreWittEquivalent( K::FldFunRat, L::FldRat ) -> BoolElt
{ " } //"
    require ISA(Category(ConstantField(K)), FldFin) : "Not implemented for these types of fields!";

    // Use Szymiczek (1991), Theorem 1.3
    return false;
end intrinsic;

