freeze;

import "../helpers/varia.m" : ZZ;

//////////////////////////////////////////////////
// IsLocalSquare

intrinsic IsLocalSquare( a::FldRatElt, P::RngInt ) -> BoolElt
{ 
Given a rational number a, decide if a is a square in the completion QQ_P
}
    return IsLocalSquare(a, Generator(P));
end intrinsic;

//////////////////////////////////////////////////
// IsSumOfSquares
// (just for completeness)
intrinsic IsSumOfSquares( a::FldRatElt ) -> BoolElt
{
Returns true iff a is a sum of squares
}
    return a ge 0;
end intrinsic;

intrinsic IsSumOfSquares( a::RngIntElt ) -> BoolElt
{ " } //"
    return a ge 0;
end intrinsic;

intrinsic IsSOS( a::FldRatElt ) -> BoolElt
{ " } //"
    return a ge 0;
end intrinsic;

intrinsic IsSOS( a::RngIntElt ) -> BoolElt
{ " } //"
    return a ge 0;
end intrinsic;


//////////////////////////////////////////////////
// LengthOfSOS

intrinsic LengthOfSOS( a::RngIntElt ) -> RngIntElt
{
Given a nonzero integer a, this fuction determines
minimal number of summands needed to express a as a sum of squares. 
It returns 0 if a is not a sum of squares.
}
    require not IsZero(a): "The argument must be nonzero!";
    
    // start with trivial cases
    if a lt 0 then
        return 0;
    end if;
    if IsSquare(a) then
        return 1;
    end if;
    // special case
    if a eq 2 then
        return 2;
    end if;
    
    // check if a is a sum of 3 squares
    // write a in a form a = 4^v'*a'
    v := Valuation(a, 2);
    v_ := Floor( v/2 );
    a_ := ZZ ! ( a*4^(-v_) );
    // use Lagrange 3-square theorem
    if (a_ mod 8) eq 7 then
        return 4;
    end if;
    
    // now a_ (hence a) is a sum of 2 or 3 squares
    // compute the radical of a
    m, _ := SquarefreeFactorization(a);
    // use Lagrange 2-square theorem
    sq2 := forall{ p : p in PrimeDivisors(m) | (p mod 4) eq 1 };
    return sq2 select 2 else 3;
end intrinsic;

intrinsic LengthOfSumOfSquares( a::RngIntElt ) -> RngIntElt
{ " } //"
    return LengthOfSOS(a);
end intrinsic;

intrinsic LengthOfSOS( a::FldRatElt ) -> RngIntElt
{
Given a nonzero rational number a, this fuction determines
minimal number of summands needed to express a as a sum of squares. 
It returns 0 if a is not a sum of squares.
}
    return LengthOfSOS( Numerator(a)*Denominator(a) );
end intrinsic;

intrinsic LengthOfSumOfSquares( a::FldRatElt ) -> RngIntElt
{ " } //"
    return LengthOfSOS(a);
end intrinsic;

